#---
# Excerpted from "The ThoughtWorks Anthology",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/twa for more book information.
#---
# using explicit receivers
# I'm using an expression builder here, but I feel that
# this style is more natural without one. 
# I just don't want t
require 'model'

class ConfigurationBuilder
  attr_reader :subject
  def initialize
    @subject = Configuration.new
  end
  def self.start
    newObj = self.new
    yield newObj
    return newObj
  end
  def item(name, &block) 
    i = ItemBuilder.new(name, subject)
    i.instance_eval(&block) if block_given?
    subject.add_item i.subject
  end
end

class ItemBuilder
  attr_reader :subject, :config
  def initialize name, config
    @subject = Item.new(name)
    @config = config
  end
  def uses(arg, &block)
    arg.instance_eval(&block) if block_given?
    @subject.add_usage arg.subject
  end
  def provides arg
    yield arg if block_given?
    @subject.add_provision arg.subject
  end
  def depends_on arg
    @subject.add_dependency @config[arg]
  end
  def acid
    return AcidBuilder.new
  end
  def electricity power
    return ElectricityBuilder.new(power)
  end
end


class AcidBuilder
  attr_reader :subject
  def initialize
    @subject = Acid.new
  end
  def type arg
    subject.type = arg
    self
  end
  def grade arg
    subject.grade = arg
    self
  end
end

class ElectricityBuilder
  attr_reader :subject
  def initialize arg
    @subject = Electricity.new arg
  end
end



class RulesBuilder18
  def load aStream
    builder = ConfigurationBuilder.new
    builder.instance_eval aStream.join("\n")
    return builder.subject
  end

end

if __FILE__ == $0
  c = RulesBuilder18.new.load(File.readlines('rules18.rb'))
  require 'pp'
#  puts c
end
